Class: Runger::Config

Inherits:
Object
  • Object
show all
Extended by:
RBSGenerator
Includes:
DynamicConfig, OptparseConfig, Rails::Config
Defined in:
lib/runger/config.rb

Overview

Base config class Provides attr_config method to describe configuration parameters and set defaults

Defined Under Namespace

Classes: BlockCallback, Error, NamedCallback, ValidationError

Constant Summary collapse

PARAM_NAME =
/^[a-z_](\w+)?$/
RESERVED_NAMES =

List of names that couldn’t be used as config names (the class instance methods we use)

%i[
  config_name
  env_prefix
  as_env
  values
  class
  clear
  deconstruct_keys
  dig
  dup
  initialize
  load
  load_from_sources
  option_parser
  pretty_print
  raise_validation_error
  reload
  resolve_config_path
  tap
  to_h
  to_source_trace
  write_config_attr
  __type_caster__
].freeze

Constants included from RBSGenerator

RBSGenerator::TYPE_TO_CLASS

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from RBSGenerator

to_rbs

Methods included from DynamicConfig

included

Methods included from OptparseConfig

included, #option_parser, #parse_options!

Constructor Details

#initialize(overrides = nil) ⇒ Config

Instantiate config instance.

Example:

my_config = Runger::Config.new()

# provide some values explicitly
my_config = Runger::Config.new({some: :value})

Raises:

  • (ArgumentError)


342
343
344
345
346
347
348
349
350
351
# File 'lib/runger/config.rb', line 342

def initialize(overrides = nil)
  @config_name = self.class.config_name

  raise(ArgumentError, 'Config name is missing') unless @config_name

  @env_prefix = self.class.env_prefix
  @values = {}

  load(overrides)
end

Instance Attribute Details

#config_nameObject (readonly)

Returns the value of attribute config_name.



331
332
333
# File 'lib/runger/config.rb', line 331

def config_name
  @config_name
end

#env_prefixObject (readonly)

Returns the value of attribute env_prefix.



331
332
333
# File 'lib/runger/config.rb', line 331

def env_prefix
  @env_prefix
end

Class Method Details

.attr_config(*args, **hargs) ⇒ Object



81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
# File 'lib/runger/config.rb', line 81

def attr_config(*args, **hargs)
  new_defaults = hargs.deep_dup
  new_defaults.stringify_keys!

  defaults.merge!(new_defaults)

  new_keys = ((args + new_defaults.keys) - config_attributes)

  validate_param_names!(new_keys.map(&:to_s))

  new_keys.map!(&:to_sym)

  unless (reserved_names = (new_keys & RESERVED_NAMES)).empty?
    raise(ArgumentError,
      'Can not use the following reserved names as config attrubutes: ' \
      "#{reserved_names.sort.map(&:to_s).join(', ')}")
  end

  config_attributes.push(*new_keys)

  define_config_accessor(*new_keys)

  # Define predicate methods ("param?") for attributes
  # having `true` or `false` as default values
  new_defaults.each do |key, val|
    next unless val.is_a?(TrueClass) || val.is_a?(FalseClass)

    alias_method(:"#{key}?", :"#{key}")
  end
end

.coerce_types(mapping) ⇒ Object



229
230
231
232
233
234
235
236
237
238
# File 'lib/runger/config.rb', line 229

def coerce_types(mapping)
  ::Runger::Utils.deep_merge!(coercion_mapping, mapping)

  mapping.each do |key, val|
    type = val.is_a?(::Hash) ? val[:type] : val
    next if type != :boolean

    alias_method(:"#{key}?", :"#{key}")
  end
end

.coercion_mappingObject



240
241
242
243
244
245
246
247
248
249
# File 'lib/runger/config.rb', line 240

def coercion_mapping
  return @coercion_mapping if instance_variable_defined?(:@coercion_mapping)

  @coercion_mapping =
    if superclass < Runger::Config
      superclass.coercion_mapping.deep_dup
    else
      {}
    end
end

.config_attributesObject



123
124
125
126
127
128
129
130
131
132
# File 'lib/runger/config.rb', line 123

def config_attributes
  return @config_attributes if instance_variable_defined?(:@config_attributes)

  @config_attributes =
    if superclass < Runger::Config
      superclass.config_attributes.dup
    else
      []
    end
end

.config_name(val = nil) ⇒ Object



182
183
184
185
186
187
188
# File 'lib/runger/config.rb', line 182

def config_name(val = nil)
  return (@explicit_config_name = val.to_s) unless val.nil?

  return @config_name if instance_variable_defined?(:@config_name)

  @config_name = explicit_config_name || build_config_name
end

.defaultsObject



112
113
114
115
116
117
118
119
120
121
# File 'lib/runger/config.rb', line 112

def defaults
  return @defaults if instance_variable_defined?(:@defaults)

  @defaults =
    if superclass < Runger::Config
      superclass.defaults.deep_dup
    else
      new_empty_config
    end
end

.disable_auto_cast!Object



275
276
277
# File 'lib/runger/config.rb', line 275

def disable_auto_cast!
  @fallback_type_caster = ::Runger::NoCast
end

.env_prefix(val = nil) ⇒ Object



201
202
203
204
205
206
207
208
209
210
211
212
# File 'lib/runger/config.rb', line 201

def env_prefix(val = nil)
  return (@env_prefix = val.to_s.upcase) unless val.nil?

  return @env_prefix if instance_variable_defined?(:@env_prefix)

  @env_prefix =
    if superclass < Runger::Config && superclass.explicit_config_name?
      superclass.env_prefix
    else
      config_name.upcase
    end
end

.explicit_config_nameObject



190
191
192
193
194
195
196
197
# File 'lib/runger/config.rb', line 190

def explicit_config_name
  return @explicit_config_name if instance_variable_defined?(:@explicit_config_name)

  @explicit_config_name =
    if superclass.respond_to?(:explicit_config_name)
      superclass.explicit_config_name
    end
end

.explicit_config_name?Boolean

Returns:

  • (Boolean)


199
# File 'lib/runger/config.rb', line 199

def explicit_config_name? = !explicit_config_name.nil?

.fallback_type_caster(val = nil) ⇒ Object



262
263
264
265
266
267
268
269
270
271
272
273
# File 'lib/runger/config.rb', line 262

def fallback_type_caster(val = nil)
  return (@fallback_type_caster = val) unless val.nil?

  return @fallback_type_caster if instance_variable_defined?(:@fallback_type_caster)

  @fallback_type_caster =
    if superclass < Runger::Config
      superclass.fallback_type_caster.deep_dup
    else
      ::Runger::AutoCast
    end
end

.load_callbacksObject



171
172
173
174
175
176
177
178
179
180
# File 'lib/runger/config.rb', line 171

def load_callbacks
  return @load_callbacks if instance_variable_defined?(:@load_callbacks)

  @load_callbacks =
    if superclass <= Runger::Config
      superclass.load_callbacks.dup
    else
      []
    end
end

.loader_options(val = nil) ⇒ Object



214
215
216
217
218
219
220
221
222
223
224
225
# File 'lib/runger/config.rb', line 214

def loader_options(val = nil)
  return (@loader_options = val) unless val.nil?

  return @loader_options if instance_variable_defined?(:@loader_options)

  @loader_options =
    if superclass < Runger::Config
      superclass.loader_options
    else
      {}
    end
end

.new_empty_configObject



227
# File 'lib/runger/config.rb', line 227

def new_empty_config = {}

.on_load(*names, &block) ⇒ Object



158
159
160
161
162
163
164
165
166
167
168
169
# File 'lib/runger/config.rb', line 158

def on_load(*names, &block)
  if block && !names.empty?
    raise(ArgumentError,
      'Either methods or block should be specified, not both')
  end

  if block
    load_callbacks << BlockCallback.new(block)
  else
    load_callbacks.push(*names.map { NamedCallback.new(_1) })
  end
end

.required(*names, env: nil, **nested) ⇒ Object



134
135
136
137
138
139
140
141
142
143
144
145
# File 'lib/runger/config.rb', line 134

def required(*names, env: nil, **nested)
  unknown_names = names + nested.keys - config_attributes
  if unknown_names.any?
    raise(ArgumentError,
      "Unknown config param: #{unknown_names.join(',')}")
  end

  return unless Settings.matching_env?(env)

  required_attributes.push(*names)
  required_attributes.push(*nested.flatten_names)
end

.required_attributesObject



147
148
149
150
151
152
153
154
155
156
# File 'lib/runger/config.rb', line 147

def required_attributes
  return @required_attributes if instance_variable_defined?(:@required_attributes)

  @required_attributes =
    if superclass < Runger::Config
      superclass.required_attributes.dup
    else
      []
    end
end

.type_caster(val = nil) ⇒ Object



251
252
253
254
255
256
257
258
259
260
# File 'lib/runger/config.rb', line 251

def type_caster(val = nil)
  return @type_caster unless val.nil?

  @type_caster ||=
    if coercion_mapping.empty?
      fallback_type_caster
    else
      ::Runger::TypeCaster.new(coercion_mapping, fallback: fallback_type_caster)
    end
end

Instance Method Details

#as_envObject



454
455
456
# File 'lib/runger/config.rb', line 454

def as_env
  Env.from_hash(to_h, prefix: env_prefix)
end

#clearObject



359
360
361
362
363
# File 'lib/runger/config.rb', line 359

def clear
  values.clear
  @__trace__ = nil
  self
end

#deconstruct_keys(keys) ⇒ Object



430
# File 'lib/runger/config.rb', line 430

def deconstruct_keys(keys) = values.deconstruct_keys(keys)

#digObject



413
# File 'lib/runger/config.rb', line 413

def dig(*) = values.dig(*)

#dupObject



417
418
419
420
421
422
423
424
# File 'lib/runger/config.rb', line 417

def dup
  self.class.allocate.tap do |new_config|
    %i[config_name env_prefix __trace__].each do |ivar|
      new_config.instance_variable_set(:"@#{ivar}", send(ivar).dup)
    end
    new_config.instance_variable_set(:@values, values.deep_dup)
  end
end

#inspectObject



434
435
436
437
438
# File 'lib/runger/config.rb', line 434

def inspect
  "#<#{self.class}:0x#{vm_object_id.rjust(16,
    '0')} config_name=\"#{config_name}\" env_prefix=\"#{env_prefix}\" " \
    "values=#{values.inspect}>"
end

#load(overrides = nil) ⇒ Object



365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
# File 'lib/runger/config.rb', line 365

def load(overrides = nil)
  base_config = self.class.defaults.deep_dup

  trace =
    Tracing.capture do
      Tracing.trace!(:defaults) { base_config }

      config_path = resolve_config_path(config_name, env_prefix)

      load_from_sources(
        base_config,
        name: config_name,
        env_prefix:,
        config_path:,
        **self.class.loader_options,
      )

      if overrides
        Tracing.trace!(:load) { overrides }

        ::Runger::Utils.deep_merge!(base_config, overrides)
      end
    end

  base_config.each do |key, val|
    write_config_attr(key.to_sym, val)
  end

  # Trace may contain unknown attributes
  trace&.keep_if { |key| self.class.config_attributes.include?(key.to_sym) }

  # Run on_load callbacks
  self.class.load_callbacks.each { _1.apply_to(self) }

  # Set trace after we write all the values to
  # avoid changing the source to accessor
  @__trace__ = trace

  self
end

#load_from_sources(base_config, **options) ⇒ Object



406
407
408
409
410
411
# File 'lib/runger/config.rb', line 406

def load_from_sources(base_config, **options)
  Runger.loaders.each do |(_id, loader)|
    ::Runger::Utils.deep_merge!(base_config, loader.call(**options))
  end
  base_config
end

#pretty_print(q) ⇒ Object



440
441
442
443
444
445
446
447
448
449
450
451
452
# File 'lib/runger/config.rb', line 440

def pretty_print(q)
  q.object_group(self) do
    q.nest(1) do
      q.breakable
      q.text "config_name=#{config_name.inspect}"
      q.breakable
      q.text "env_prefix=#{env_prefix.inspect}"
      q.breakable
      q.text 'values:'
      q.pp(__trace__)
    end
  end
end

#reload(overrides = nil) ⇒ Object



353
354
355
356
357
# File 'lib/runger/config.rb', line 353

def reload(overrides = nil)
  clear
  load(overrides)
  self
end

#resolve_config_path(name, env_prefix) ⇒ Object



426
427
428
# File 'lib/runger/config.rb', line 426

def resolve_config_path(name, env_prefix)
  Runger.env.fetch(env_prefix).delete('conf') || Settings.default_config_path.call(name)
end

#to_hObject



415
# File 'lib/runger/config.rb', line 415

def to_h = values.deep_dup.deep_freeze

#to_source_traceObject



432
# File 'lib/runger/config.rb', line 432

def to_source_trace = __trace__&.to_h