Module: Runger::Rails::Config::ClassMethods
- Defined in:
- lib/runger/rails/config.rb
Overview
Enhance config to be more Railsy-like: – accept hashes with indeferent access
-
load data from secrets
-
recognize Rails env when loading from YML
Instance Method Summary collapse
-
#new_empty_config ⇒ Object
Make defaults to be a Hash with indifferent access.
Instance Method Details
#new_empty_config ⇒ Object
Make defaults to be a Hash with indifferent access
13 14 15 |
# File 'lib/runger/rails/config.rb', line 13 def new_empty_config {}.with_indifferent_access end |