Module: Runger::Rails
- Defined in:
- lib/runger/rails/autoload.rb,
lib/runger/rails.rb
Overview
This module is used to detect a Rails application and activate the corresponding plugins when Runger Config is loaded before Rails (e.g., in config/puma.rb).
Defined Under Namespace
Class Attribute Summary collapse
-
.disable_postponed_load_warning ⇒ Object
Returns the value of attribute disable_postponed_load_warning.
-
.name_method ⇒ Object
readonly
Returns the value of attribute name_method.
-
.tracer ⇒ Object
readonly
Returns the value of attribute tracer.
Class Attribute Details
.disable_postponed_load_warning ⇒ Object
Returns the value of attribute disable_postponed_load_warning.
8 9 10 |
# File 'lib/runger/rails/autoload.rb', line 8 def disable_postponed_load_warning @disable_postponed_load_warning end |
.name_method ⇒ Object (readonly)
Returns the value of attribute name_method.
7 8 9 |
# File 'lib/runger/rails/autoload.rb', line 7 def name_method @name_method end |
.tracer ⇒ Object (readonly)
Returns the value of attribute tracer.
7 8 9 |
# File 'lib/runger/rails/autoload.rb', line 7 def tracer @tracer end |